/*
 * Copyright (C) 2003-2009 eXo Platform SAS.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.reflext.api.annotation;

import org.reflext.api.ClassTypeInfo;

import java.lang.annotation.Annotation;

/**
 * The type of annotation.
 *
 * @author <a href="mailto:julien.viet@exoplatform.com">Julien Viet</a>
 * @version $Revision$
 */
public abstract class AnnotationType<A, T> {

  public static <A extends Annotation> AnnotationType<A, Class<A>> get(Class<A> a) {
    return new Runtime<A>(a);
  }

  public static AnnotationType<AnnotationInfo, ClassTypeInfo> get(ClassTypeInfo a) {
    return new Meta(a);
  }

  private static class Runtime<A extends Annotation> extends AnnotationType<A, Class<A>> {
    private final Class<A> type;
    private Runtime(Class<A> type) {
      this.type = type;
    }
    @Override
    public Class<A> getType() {
      return type;
    }
  }

  private static class Meta extends AnnotationType<AnnotationInfo, ClassTypeInfo> {
    private final ClassTypeInfo type;
    private Meta(ClassTypeInfo type) {
      this.type = type;
    }
    @Override
    public ClassTypeInfo getType() {
      return type;
    }
  }

  protected AnnotationType() {
  }

  public abstract T getType();

}
